OML = DataFrame(name=String[], 
                id=Int[],
                response_var=Symbol[], 
                ignore=Array{Symbol}[], 
                coercion_rules=Dict[],
            )
push!(OML, ["fried", 564, :Y, [], Dict()])
push!(OML, ["stock", 1200, :company10, [], Dict()])
push!(OML, ["mlr_knn_rng", 42454, Symbol("perf.logloss"), [Symbol("perf.mmce")], Dict(:distance=>Multiclass)])
push!(OML, ["methane", 42701, :MM264, [:MM263,:MM256,:year], Dict()])
push!(OML, ["diamonds", 42225, :price, [], Dict()])
push!(OML, ["protein", 42903, :RMSD, [], Dict()])
push!(OML, ["house_8L", 218, :price, [], Dict()])
push!(OML, ["sulfur", 44145, :y1, [], Dict()])
push!(OML, ["elevators", 216, :Goal, [], Dict()])
push!(OML, ["sarcos", 43873, :V22, [:V23,:V24,:V25,:V26,:V27,:V28], Dict()])

function OML_row(name) 
    r = findfirst(==(name), OML.name)
    @assert ~isnothing(r) "Unrecognized dataset '$(name)'. Supported values are: $(OML.name)."
    return r
end
OML_id(name) = OML.id[OML_row(name)]

function load_openml_dataset_table(name)
    openml_id = OML_id(name)
    T = OpenML.load(openml_id) # Something Tables.jl-compatible
    return T
end

"""
$(SIGNATURES)    
Returns data (X,y) of the OpenML dataset with name `name` (see table `OML` for supported datasets). The following operations will be performed:
- Entries with missing values are dropped. 
- Coercion rules specified in `OML.coercion_rules` apply
- A `ContinuousEncoder` is used
- Standardization is performed according to `kwargs`
- X,y are separated using `unpack`.
"""
function load_openml_dataset(::Type{DataFrame}, name; kwargs...)
    T = load_openml_dataset_table(name)
    id = OML_row(name)
    resp_var = OML.response_var[id]
    # @show schema(T)
    @info "Loading dataset $(name), response variable is $(resp_var)."
    # Not clear what we loose here… Might want to postpone that for datasets that don't fit in memory?
    df = DataFrame(T)
    df, _ = unpack(df, x->~in(x, OML.ignore[id]))
    dropmissing!(df)
    df = custom_coerce(df, OML.coercion_rules[id])
    df = standardize(df, resp_var; kwargs...)
    X, y = custom_unpack(df, resp_var)
    return (X,y)
end

load_openml_dataset(name; kwargs...) = load_dataset(Matrix, name; kwargs...)
function download_OpenML_datasets(; kwargs...)
    for n in OML.name
        _ = load_openml_dataset(n; kwargs...)
    end
end

function custom_unpack(T, resp_var)
    y, X = unpack(T, ==(resp_var))
    @assert (size(X,2) > 0) "Got size(X)=$(size(X)). Check that index '$(resp_var)' does correspond to a valid feature name and is provided as a Symbol. Valid indexes are: $(names(T))."
    @assert (size(y,2) == 1) "Got size(y)=$(size(y)), but expecting exactly one column."
    return X, y
end

